@@ -26,33 +26,92 @@ if( !isset( $_COOKIE[ 'security' ] ) || !in_array( $_COOKIE[ 'security' ], $secu
 	} else {
 		dvwaSecurityLevelSet( 'impossible' );
 	}
+	// If the cookie wasn't set then the session flags need updating.
+	dvwa_start_session();
 }
 
-// This will setup the session cookie based on
-// the security level.
+/*
+ * This function is called after login and when you change the security level.
+ * It gets the security level and sets the httponly and samesite cookie flags
+ * appropriately.
+ *
+ * To force an update of the cookie flags we need to update the session id,
+ * just setting the flags and doing a session_start() does not change anything.
+ * For this, session_id() or session_regenerate_id() can be used.
+ * Both keep the existing session values, so nothing is lost,
+ * it will just cause a new Set-Cookie header to be sent with the new right
+ * flags and the new id (or the same one if we wish to keep it).
+*/
+function dvwa_start_session() {
+	// This will setup the session cookie based on
+	// the security level.
 
-if (dvwaSecurityLevelGet() == 'impossible') {
-	$httponly = true;
-	$samesite = true;
-}
-else {
-	$httponly = false;
-	$samesite = false;
+	$security_level = dvwaSecurityLevelGet();
+	if ($security_level == 'impossible') {
+		$httponly = true;
+		$samesite = "Strict";
+	}
+	else {
+		$httponly = false;
+		$samesite = "";
+	}
+
+	$maxlifetime = 86400;
+	$secure = false;
+	$domain = parse_url($_SERVER['HTTP_HOST'], PHP_URL_HOST);
+
+	/*
+	 * Need to do this as you can't update the settings of a session
+	 * while it is open. So check if one is open, close it if needed
+	 * then update the values and start it again.
+	*/
+	if (session_status() == PHP_SESSION_ACTIVE) {
+		session_write_close();
+	}
+
+	session_set_cookie_params([
+		'lifetime' => $maxlifetime,
+		'path' => '/',
+		'domain' => $domain,
+		'secure' => $secure,
+		'httponly' => $httponly,
+		'samesite' => $samesite
+	]);
+
+	/*
+	 * We need to force a new Set-Cookie header with the updated flags by updating
+	 * the session id, either regenerating it or setting it to a value, because
+	 * session_start() might not generate a Set-Cookie header if a cookie already
+	 * exists.
+	 *
+	 * For impossible security level, we regenerate the session id, PHP will
+	 * generate a new random id. This is good security practice because it
+	 * prevents the reuse of a previous unauthenticated id that an attacker
+	 * might have knowledge of (aka session fixation attack).
+   *
+	 * For lower levels, we want to allow session fixation attacks, so if an id
+	 * already exists, we don't want it to change after authentication. We thus
+	 * set the id to its previous value using session_id(), which will force
+	 * the Set-Cookie header.
+	*/
+	if ($security_level == 'impossible') {
+		session_start();
+		session_regenerate_id(); // force a new id to be generated
+	}
+	else {
+		if (isset($_COOKIE[session_name()])) // if a session id already exists
+			session_id($_COOKIE[session_name()]); // we keep the same id
+		session_start(); // otherwise a new one will be generated here
+	}
 }
 
-$maxlifetime = 86400;
-$secure = false;
-$domain = parse_url($_SERVER['HTTP_HOST'], PHP_URL_HOST);
-
-session_set_cookie_params([
-	'lifetime' => $maxlifetime,
-	'path' => '/',
-	'domain' => $domain,
-	'secure' => $secure,
-	'httponly' => $httponly,
-	'samesite' => $samesite
-]);
-session_start();
+if (array_key_exists ("Login", $_POST) && $_POST['Login'] == "Login") {
+	dvwa_start_session();
+} else {
+	if (!session_id()) {
+		session_start();
+	}
+}
 
 if (!array_key_exists ("default_locale", $_DVWA)) {
 	$_DVWA[ 'default_locale' ] = "en";
@@ -60,17 +119,6 @@ if (!array_key_exists ("default_locale", $_DVWA)) {
 
 dvwaLocaleSet( $_DVWA[ 'default_locale' ] );
 
-// DVWA version
-function dvwaVersionGet() {
-	return '1.10 *Development*';
-}
-
-// DVWA release date
-function dvwaReleaseDateGet() {
-	return '2015-10-08';
-}
-
-
 // Start session functions --
 
 function &dvwaSessionGrab() {
@@ -113,7 +161,12 @@ function dvwaLogout() {
 
 
 function dvwaPageReload() {
-	dvwaRedirect( $_SERVER[ 'PHP_SELF' ] );
+	if  ( array_key_exists( 'HTTP_X_FORWARDED_PREFIX' , $_SERVER )) {
+		dvwaRedirect( $_SERVER[ 'HTTP_X_FORWARDED_PREFIX' ] . $_SERVER[ 'PHP_SELF' ] );
+	}
+	else {
+		dvwaRedirect( $_SERVER[ 'PHP_SELF' ] );
+	}
 }
 
 function dvwaCurrentUser() {
@@ -164,9 +217,10 @@ function dvwaSecurityLevelSet( $pSecurityLevel ) {
 	}
 
 	setcookie( 'security', $pSecurityLevel, 0, "/", "", false, $httponly );
+	$_COOKIE['security'] = $pSecurityLevel;
 }
 
-function dvwaLocaleGet() {	
+function dvwaLocaleGet() {
 	$dvwaSession =& dvwaSessionGrab();
 	return $dvwaSession[ 'locale' ];
 }
@@ -251,6 +305,7 @@ function dvwaHtmlEcho( $pPage ) {
 			$menuBlocks[ 'vulnerabilities' ][] = array( 'id' => 'authbypass', 'name' => 'Authorisation Bypass', 'url' => 'vulnerabilities/authbypass/' );
 		}
 		$menuBlocks[ 'vulnerabilities' ][] = array( 'id' => 'open_redirect', 'name' => 'Open HTTP Redirect', 'url' => 'vulnerabilities/open_redirect/' );
+		$menuBlocks[ 'vulnerabilities' ][] = array( 'id' => 'encryption', 'name' => 'Cryptography', 'url' => 'vulnerabilities/cryptography/' );
 	}
 
 	$menuBlocks[ 'meta' ] = array();
@@ -299,7 +354,7 @@ function dvwaHtmlEcho( $pPage ) {
 	$securityLevelHtml = "<em>Security Level:</em> {$securityLevelHtml}";
 	$localeHtml = '<em>Locale:</em> ' . ( dvwaLocaleGet() );
 	$sqliDbHtml = '<em>SQLi DB:</em> ' . ( dvwaSQLiDBGet() );
-	
+
 
 	$messagesHtml = messagesPopAllToHtml();
 	if( $messagesHtml ) {
@@ -307,7 +362,7 @@ function dvwaHtmlEcho( $pPage ) {
 	}
 
 	$systemInfoHtml = "";
-	if( dvwaIsLoggedIn() ) 
+	if( dvwaIsLoggedIn() )
 		$systemInfoHtml = "<div align=\"left\">{$userInfoHtml}<br />{$securityLevelHtml}<br />{$localeHtml}<br />{$sqliDbHtml}</div>";
 	if( $pPage[ 'source_button' ] ) {
 		$systemInfoHtml = dvwaButtonSourceHtmlGet( $pPage[ 'source_button' ] ) . " $systemInfoHtml";
@@ -458,7 +513,7 @@ function dvwaSourceHtmlEcho( $pPage ) {
 
 // To be used on all external links --
 function dvwaExternalLinkUrlGet( $pLink,$text=null ) {
-	if(is_null( $text )) {
+	if(is_null( $text ) || $text == "") {
 		return '<a href="' . $pLink . '" target="_blank">' . $pLink . '</a>';
 	}
 	else {
@@ -484,24 +539,14 @@ function dvwaButtonSourceHtmlGet( $pId ) {
 
 if( $DBMS == 'MySQL' ) {
 	$DBMS = htmlspecialchars(strip_tags( $DBMS ));
-	$DBMS_errorFunc = 'mysqli_error()';
 }
 elseif( $DBMS == 'PGSQL' ) {
 	$DBMS = htmlspecialchars(strip_tags( $DBMS ));
-	$DBMS_errorFunc = 'pg_last_error()';
 }
 else {
 	$DBMS = "No DBMS selected.";
-	$DBMS_errorFunc = '';
 }
 
-//$DBMS_connError = '
-//	<div align="center">
-//		<img src="' . DVWA_WEB_PAGE_TO_ROOT . 'dvwa/images/logo.png" />
-//		<pre>Unable to connect to the database.<br />' . $DBMS_errorFunc . '<br /><br /></pre>
-//		Click <a href="' . DVWA_WEB_PAGE_TO_ROOT . 'setup.php">here</a> to setup the database.
-//	</div>';
-
 function dvwaDatabaseConnect() {
 	global $_DVWA;
 	global $DBMS;
@@ -514,7 +559,7 @@ function dvwaDatabaseConnect() {
 		|| !@((bool)mysqli_query($GLOBALS["___mysqli_ston"], "USE " . $_DVWA[ 'db_database' ])) ) {
 			//die( $DBMS_connError );
 			dvwaLogout();
-			dvwaMessagePush( 'Unable to connect to the database.<br />' . $DBMS_errorFunc );
+			dvwaMessagePush( 'Unable to connect to the database.<br />' . mysqli_error($GLOBALS["___mysqli_ston"]));
 			dvwaRedirect( DVWA_WEB_PAGE_TO_ROOT . 'setup.php' );
 		}
 		// MySQL PDO Prepared Statements (for impossible levels)
@@ -609,9 +654,9 @@ $PHPUploadPath    = realpath( getcwd() . DIRECTORY_SEPARATOR . DVWA_WEB_PAGE_TO_
 $PHPCONFIGPath       = realpath( getcwd() . DIRECTORY_SEPARATOR . DVWA_WEB_PAGE_TO_ROOT . "config");
 
 
-$phpDisplayErrors = 'PHP function display_errors: <em>' . ( ini_get( 'display_errors' ) ? 'Enabled</em> <i>(Easy Mode!)</i>' : 'Disabled</em>' );                                                  // Verbose error messages (e.g. full path disclosure)
-$phpSafeMode      = 'PHP function safe_mode: <span class="' . ( ini_get( 'safe_mode' ) ? 'failure">Enabled' : 'success">Disabled' ) . '</span>';                                                   // DEPRECATED as of PHP 5.3.0 and REMOVED as of PHP 5.4.0
-$phpMagicQuotes   = 'PHP function magic_quotes_gpc: <span class="' . ( ini_get( 'magic_quotes_gpc' ) ? 'failure">Enabled' : 'success">Disabled' ) . '</span>';                                     // DEPRECATED as of PHP 5.3.0 and REMOVED as of PHP 5.4.0
+$phpDisplayErrors = 'PHP function display_errors: <span class="' . ( ini_get( 'display_errors' ) ? 'success">Enabled' : 'failure">Disabled' ) . '</span>';                                                  // Verbose error messages (e.g. full path disclosure)
+$phpDisplayStartupErrors = 'PHP function display_startup_errors: <span class="' . ( ini_get( 'display_startup_errors' ) ? 'success">Enabled' : 'failure">Disabled' ) . '</span>';                                                  // Verbose error messages (e.g. full path disclosure)
+$phpDisplayErrors = 'PHP function display_errors: <span class="' . ( ini_get( 'display_errors' ) ? 'success">Enabled' : 'failure">Disabled' ) . '</span>';                                                  // Verbose error messages (e.g. full path disclosure)
 $phpURLInclude    = 'PHP function allow_url_include: <span class="' . ( ini_get( 'allow_url_include' ) ? 'success">Enabled' : 'failure">Disabled' ) . '</span>';                                   // RFI
 $phpURLFopen      = 'PHP function allow_url_fopen: <span class="' . ( ini_get( 'allow_url_fopen' ) ? 'success">Enabled' : 'failure">Disabled' ) . '</span>';                                       // RFI
 $phpGD            = 'PHP module gd: <span class="' . ( ( extension_loaded( 'gd' ) && function_exists( 'gd_info' ) ) ? 'success">Installed' : 'failure">Missing - Only an issue if you want to play with captchas' ) . '</span>';                    // File Upload
@@ -619,8 +664,8 @@ $phpMySQL         = 'PHP module mysql: <span class="' . ( ( extension_loaded( 'm
 $phpPDO           = 'PHP module pdo_mysql: <span class="' . ( extension_loaded( 'pdo_mysql' ) ? 'success">Installed' : 'failure">Missing' ) . '</span>';                // SQLi
 $DVWARecaptcha    = 'reCAPTCHA key: <span class="' . ( ( isset( $_DVWA[ 'recaptcha_public_key' ] ) && $_DVWA[ 'recaptcha_public_key' ] != '' ) ? 'success">' . $_DVWA[ 'recaptcha_public_key' ] : 'failure">Missing' ) . '</span>';
 
-$DVWAUploadsWrite = '[User: ' . get_current_user() . '] Writable folder ' . $PHPUploadPath . ': <span class="' . ( is_writable( $PHPUploadPath ) ? 'success">Yes' : 'failure">No' ) . '</span>';                                     // File Upload
-$bakWritable = '[User: ' . get_current_user() . '] Writable folder ' . $PHPCONFIGPath . ': <span class="' . ( is_writable( $PHPCONFIGPath ) ? 'success">Yes' : 'failure">No' ) . '</span>';   // config.php.bak check                                  // File Upload
+$DVWAUploadsWrite = 'Writable folder ' . $PHPUploadPath . ': <span class="' . ( is_writable( $PHPUploadPath ) ? 'success">Yes' : 'failure">No' ) . '</span>';                                     // File Upload
+$bakWritable = 'Writable folder ' . $PHPCONFIGPath . ': <span class="' . ( is_writable( $PHPCONFIGPath ) ? 'success">Yes' : 'failure">No' ) . '</span>';   // config.php.bak check                                  // File Upload
 
 $DVWAOS           = 'Operating system: <em>' . ( strtoupper( substr (PHP_OS, 0, 3)) === 'WIN' ? 'Windows' : '*nix' ) . '</em>';
 $SERVER_NAME      = 'Web Server SERVER_NAME: <em>' . $_SERVER[ 'SERVER_NAME' ] . '</em>';                                                                                                          // CSRF
