@@ -12,7 +12,8 @@ if( isset( $_GET[ 'Change' ] ) ) {
 		$pass_new = md5( $pass_new );
 
 		// Update the database
-		$insert = "UPDATE `users` SET password = '$pass_new' WHERE user = '" . dvwaCurrentUser() . "';";
+		$current_user = dvwaCurrentUser();
+		$insert = "UPDATE `users` SET password = '$pass_new' WHERE user = '" . $current_user . "';";
 		$result = mysqli_query($GLOBALS["___mysqli_ston"],  $insert ) or die( '<pre>' . ((is_object($GLOBALS["___mysqli_ston"])) ? mysqli_error($GLOBALS["___mysqli_ston"]) : (($___mysqli_res = mysqli_connect_error()) ? $___mysqli_res : false)) . '</pre>' );
 
 		// Feedback for the user
