@@ -8,7 +8,7 @@ if( array_key_exists( "name", $_GET ) && $_GET[ 'name' ] != NULL ) {
 	$name = str_replace( '<script>', '', $_GET[ 'name' ] );
 
 	// Feedback for end user
-	$html .= "<pre>Hello ${name}</pre>";
+	$html .= "<pre>Hello {$name}</pre>";
 }
 
 ?>
